@@ -68,6 +68,7 @@ struct _BamfLegacyWindowTest
   GdkRectangle geometry;
   BamfWindowMaximizationType maximized;
   BamfWindowType window_type;
+  GHashTable * hints;
 };
 
 struct _BamfLegacyWindowTestClass
@@ -116,7 +117,7 @@ gboolean
 bamf_legacy_window_test_is_skip_tasklist (BamfLegacyWindow *legacy_window);
 
 void
-bamf_legacy_window_test_set_name (BamfLegacyWindowTest *self, char *val);
+bamf_legacy_window_test_set_name (BamfLegacyWindowTest *self, const char *val);
 
 void
 bamf_legacy_window_test_set_geometry (BamfLegacyWindowTest *self, int x, int y,
@@ -135,11 +136,14 @@ bamf_legacy_window_test_set_unique_bus_name (BamfLegacyWindowTest *self, const c
 void
 bamf_legacy_window_test_set_dbus_menu_object_path (BamfLegacyWindowTest *self, const char *object_path);
 
+void
+bamf_legacy_window_test_set_wmclass (BamfLegacyWindowTest *self, const char *class_name, const char *instance_name);
+
 void
 bamf_legacy_window_test_close (BamfLegacyWindowTest *self);
 
 BamfLegacyWindowTest *
-bamf_legacy_window_test_new (guint32 xid, gchar *name, gchar *wmclass_name, gchar *exec);
+bamf_legacy_window_test_new (guint32 xid, const gchar *name, const gchar *wmclass_name, const gchar *exec);
 
 BamfLegacyWindowTest *
 bamf_legacy_window_copy (BamfLegacyWindowTest *self);
