@@ -76,7 +76,7 @@ time_t             bamf_window_opened (BamfWindow *window);
 
 gint               bamf_window_get_stack_position (BamfWindow *window);
 
-char             * bamf_window_get_xprop (BamfWindow *self, const char* prop);
+char             * bamf_window_get_string_hint (BamfWindow *self, const char* prop);
 
 BamfWindowMaximizationType bamf_window_maximized (BamfWindow *self);
 
