@@ -26,8 +26,8 @@
 #include <X11/Xlib.h>
 #include <gdk/gdkx.h>
 
-void  bamf_xutils_set_window_hint (Window xid, const char *atom_name, Atom type, const char *data);
-char* bamf_xutils_get_window_hint (Window xid, const char *atom_name, Atom type);
+void  bamf_xutils_set_string_window_hint (Window xid, const char *atom_name, const char *value);
+char* bamf_xutils_get_string_window_hint (Window xid, const char *atom_name);
 
 void  bamf_xutils_get_window_class_hints (Window xid, char **class_instance_name, char **class_name);
 
